# TKE - Advanced Programmer's Editor
# Copyright (C) 2014-2019  Trevor Williams (phase1geo@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

######################################################################
# Name:    snip_parser.tac
# Author:  Trevor Williams  (phase1geo@gmail.com)
# Date:    8/10/2015
# Brief:   Parser for snippet syntax.
######################################################################

source [file join $::tke_dir lib snip_lexer.tcl]

set snip_txtt   ""
set snip_value  ""
set snip_errmsg ""
set snip_errstr ""
set snip_pos    0

proc get_retval {args} {
  return [join $args {}]
}

proc merge_values {val1 val2} {
  if {[lindex $val1 end] eq ""} {
    lset val1 end-1 "[lindex $val1 end-1][lindex $val2 0]"
    return "[lrange $val1 0 end-1] [list [lindex $val2 1]]"
  }
  return [concat $val1 $val2]
}

proc apply_tabstop {ts_val ts_tag} {
  set retval [list]
  foreach {val tag} $ts_val {
    lappend retval $val [concat $tag $ts_tag]
  }
  return $retval
}

proc parse_format {str matches} {

  FORMAT__FLUSH_BUFFER

  # Insert the string to scan
  format__scan_string $str

  # Initialize some values
  set ::format_txtt    $::snip_txtt
  set ::format_begpos  0
  set ::format_endpos  0
  set ::format_matches $matches

  # Parse the string
  if {[catch { format_parse } rc] || ($rc != 0)} {
    snippets::display_error $str $::format_errstr $::format_errmsg
    return ""
  }

  return $::format_value

}


######
# Begin autogenerated taccle (version 1.1) routines.
# Although taccle itself is protected by the GNU Public License (GPL)
# all user-supplied functions are protected by their respective
# author's license.  See http://mini.net/tcl/taccle for other details.
######

proc SNIP_ABORT {} {
    return -code return 1
}

proc SNIP_ACCEPT {} {
    return -code return 0
}

proc snip_clearin {} {
    upvar snip_token t
    set t ""
}

proc snip_error {s} {
    puts stderr $s
}

proc snip_setupvalues {stack pointer numsyms} {
    upvar 1 1 y
    set y {}
    for {set i 1} {$i <= $numsyms} {incr i} {
        upvar 1 $i y
        set y [lindex $stack $pointer]
        incr pointer
    }
}

proc snip_unsetupvalues {numsyms} {
    for {set i 1} {$i <= $numsyms} {incr i} {
        upvar 1 $i y
        unset y
    }
}

array set ::snip_table {
  56:257 reduce
  56:258 reduce
  63:266,target 53
  56:260 reduce
  123::,target 82
  46:/,target 81
  146:271,target 103
  56:266 reduce
  56:267 reduce
  1:258,target 73
  56:270 reduce
  56:271 reduce
  24:267,target 12
  126:258,target 87
  34::,target 65
  36:269,target 60
  36:270,target 60
  67:258,target 50
  138:261,target 101
  16:257,target 10
  95:?,target 97
  1:/,target 73
  28:260,target 5
  74:/,target 86
  62:257 shift
  100:262,target 90
  62:258 shift
  62:260 shift
  112:264,target 104
  62:266 shift
  31:`,target 2
  62:267 shift
  124:266,target 83
  62:270 shift
  62:271 shift
  62::,target 76
  136:268,target 19
  3:258,target 76
  77:/ reduce
  38:269,target 58
  38:270,target 58
  77:: reduce
  69:258,target 56
  70:258,target 54
  67:257 reduce
  67:258 reduce
  67:260 reduce
  77:? reduce
  18:257,target 8
  67:266 reduce
  67:267 reduce
  67:269 reduce
  31:260,target 37
  102:262,target 102
  1:266,target 73
  114:264,target 104
  115:?,target 120
  126:266,target 87
  67:266,target 50
  22:/ reduce
  5:258,target 72
  73:257 shift
  73:258 shift
  86:`,target 45
  73:260 shift
  28:267,target 5
  100:270,target 90
  22:: reduce
  73:266 shift
  41:269,target 67
  41:270,target 67
  73:267 shift
  22:? reduce
  72:258,target 55
  73:270 shift
  143:261,target 101
  73:271 shift
  143:?,target 97
  33:260,target 61
  104:262,target 102
  3:266,target 76
  78:/ reduce
  116:264,target 104
  140:0,target 14
  54:?,target 37
  78:257 reduce
  78:258 reduce
  78:260 reduce
  109::,target 95
  110::,target 96
  33:/,target 61
  69:266,target 56
  70:266,target 54
  141:268,target 18
  78:: reduce
  22:` reduce
  78:266 reduce
  78:267 reduce
  78:? reduce
  78:270 reduce
  7:258,target 69
  78:271 reduce
  75:284,target 109
  31:267,target 39
  102:270,target 107
  133:258,target 15
  43:269,target 63
  43:270,target 63
  74:258,target 65
  145:261,target 101
  8:?,target 70
  82:?,target 23
  23:257,target 36
  98:263,target 93
  61:/,target 36
  84:257 reduce
  35:260,target 64
  84:258 reduce
  23:/ shift
  84:260 reduce
  106:262,target 92
  5:266,target 72
  5:0,target 72
  84:266 reduce
  17:`,target 9
  84:267 reduce
  84:270 reduce
  84:271 reduce
  23:: shift
  48::,target 52
  72:266,target 55
  23:? shift
  9:258,target 71
  96:271,target 17
  33:267,target 61
  104:270,target 107
  88:/,target 41
  135:258,target 89
  76:258,target 29
  79:/ reduce
  80:/ reduce
  89:257 reduce
  90:257 reduce
  89:258 reduce
  90:258 reduce
  89:260 reduce
  90:260 reduce
  89:266 reduce
  90:266 reduce
  25:257,target 20
  3::,target 76
  89:267 reduce
  90:267 reduce
  76::,target 29
  79:: reduce
  80:: reduce
  89:269 reduce
  90:269 reduce
  23:` shift
  37:260,target 57
  108:262,target 102
  79:? reduce
  80:? reduce
  7:266,target 69
  121:264,target 85
  102:?,target 97
  133:266,target 15
  74:266,target 89
  0:`,target 4
  95:257 shift
  138:284,target 109
  95:258 shift
  13:?,target 75
  95:260 shift
  98:271,target 93
  95:261 shift
  95:262 shift
  95:263 shift
  35:267,target 64
  95:264 shift
  106:270,target 92
  95:266 shift
  24:/ reduce
  95:267 shift
  24:0 reduce
  95:270 shift
  78:258,target 24
  10:0,target 78
  129:?,target 100
  130:?,target 101
  24:: reduce
  27:257,target 16
  95:283 goto
  108:/,target 119
  95:284 goto
  24:? reduce
  95:285 goto
  39:260,target 59
  40:260,target 66
  9:266,target 71
  123:264,target 82
  41:?,target 67
  135:266,target 89
  20:/,target 11
  19:/,target 33
  45:277,target 48
  76:266,target 29
  81:/ reduce
  37:0,target 57
  81:: reduce
  37:267,target 57
  136:/,target 19
  108:270,target 107
  24:` reduce
  140:258,target 14
  81:? reduce
  49:270,target 60
  81:258,target 22
  102:285,target 110
  74:274,target 92
  29:257,target 4
  30:257,target 3
  124::,target 83
  47:/,target 49
  101:260,target 100
  11:271,target 79
  42:260,target 62
  102:257 shift
  102:258 shift
  113:262,target 97
  102:260 shift
  102:261 shift
  102:262 shift
  102:263 shift
  102:264 shift
  125:264,target 84
  102:266 shift
  102:267 shift
  102:270 shift
  35::,target 64
  25:/ reduce
  25:0 reduce
  78:266,target 24
  91:268,target 13
  0:/ shift
  102:283 goto
  143:284,target 109
  25:: reduce
  96:?,target 17
  102:284 goto
  102:285 goto
  2:/,target 77
  25:? reduce
  39:267,target 59
  40:267,target 66
  142:258,target 122
  0:: shift
  83:258,target 26
  0:? shift
  104:285,target 110
  32:`,target 6
  95:261,target 101
  32:257,target 6
  103:260,target 100
  13:271,target 75
  82:/ reduce
  44:260,target 68
  115:262,target 102
  127:264,target 88
  82:: reduce
  140:266,target 14
  25:` reduce
  82:? reduce
  81:266,target 22
  0:` shift
  74:282,target 94
  145:284,target 126
  113:257 reduce
  101:267,target 106
  113:258 reduce
  113:260 reduce
  0:272,target 14
  113:261 reduce
  91::,target 13
  113:262 reduce
  42:267,target 62
  113:263 reduce
  113:264 reduce
  113:270,target 97
  113:265 reduce
  144:258,target 102
  113:266 reduce
  113:267 reduce
  54:270,target 37
  113:270 reduce
  113:271 reduce
  116:?,target 120
  97:261,target 94
  34:257,target 65
  105:260,target 91
  15:271,target 13
  46:260,target 81
  117:262,target 102
  1:/ reduce
  1:0 reduce
  87:`,target 44
  27:?,target 16
  129:264,target 100
  130:264,target 101
  142:266,target 124
  118:257 shift
  1:: reduce
  83:266,target 26
  1:? reduce
  24:0,target 12
  144:?,target 102
  103:267,target 106
  123:/,target 82
  44:267,target 68
  83:/ reduce
  115:270,target 107
  146:258,target 103
  56:270,target 34
  141:0,target 18
  87:258,target 44
  108:285,target 127
  55:?,target 33
  83:: reduce
  111::,target 50
  34:/,target 65
  36:257,target 60
  83:? reduce
  17:271,target 9
  124:257 reduce
  124:258 reduce
  124:260 reduce
  100:/ reduce
  120:262,target 86
  124:261 reduce
  1:` reduce
  124:262 reduce
  124:263 reduce
  124:264 reduce
  124:265 reduce
  124:266 reduce
  124:267 reduce
  124:270 reduce
  22::,target 80
  124:271 reduce
  144:266,target 102
  100:: reduce
  100:? reduce
  9:?,target 71
  83:?,target 26
  27:/ reduce
  105:267,target 91
  27:0 reduce
  62:/,target 75
  46:267,target 81
  117:270,target 107
  2:/ reduce
  58:270,target 31
  2:0 reduce
  129:257 reduce
  130:257 reduce
  27:: reduce
  89:258,target 42
  90:258,target 43
  129:258 reduce
  130:258 reduce
  18:`,target 8
  129:260 reduce
  130:260 reduce
  129:261 reduce
  130:261 reduce
  129:262 reduce
  130:262 reduce
  129:263 reduce
  130:263 reduce
  27:? reduce
  129:264 reduce
  130:264 reduce
  129:265 reduce
  130:265 reduce
  49::,target 53
  129:266 reduce
  130:266 reduce
  129:267 reduce
  130:267 reduce
  2:: reduce
  38:257,target 58
  109:260,target 95
  110:260,target 96
  129:270 reduce
  130:270 reduce
  129:271 reduce
  130:271 reduce
  20:271,target 11
  19:271,target 43
  2:? reduce
  51:260,target 57
  3:257 reduce
  3:258 reduce
  3:260 reduce
  75:264,target 104
  84:/ reduce
  89:/,target 42
  90:/,target 43
  146:266,target 103
  87:266,target 44
  3:266 reduce
  3:267 reduce
  3:268 reduce
  3:269 reduce
  3:270 reduce
  3:271 reduce
  46:`,target 81
  84:: reduce
  135:257 reduce
  135:258 reduce
  27:` reduce
  4::,target 2
  135:260 reduce
  135:261 reduce
  77::,target 28
  84:? reduce
  135:262 reduce
  135:263 reduce
  135:264 reduce
  135:265 reduce
  119:269,target 133
  120:270,target 86
  135:266 reduce
  135:267 reduce
  2:` reduce
  101:283,target 114
  135:270 reduce
  61:270,target 36
  135:271 reduce
  92:258,target 48
  103:?,target 97
  20:257 reduce
  19:257 shift
  20:258 reduce
  19:258 shift
  20:260 reduce
  19:260 shift
  100:257,target 90
  10:268,target 78
  8:257 reduce
  41:257,target 67
  8:258 reduce
  112:260,target 123
  20:266 reduce
  19:266 shift
  8:260 reduce
  22:271,target 80
  20:267 reduce
  19:267 shift
  101:? shift
  20:268 reduce
  19:268 shift
  20:270 reduce
  20:269 reduce
  19:270 shift
  19:269 shift
  53:260,target 38
  20:271 reduce
  19:271 shift
  1:`,target 73
  124:262,target 83
  8:266 reduce
  74:`,target 4
  8:267 reduce
  8:268 reduce
  28:/ reduce
  19:276 goto
  8:270 reduce
  8:269 reduce
  28:0 reduce
  8:271 reduce
  141:257 reduce
  141:258 reduce
  141:260 reduce
  3:/ reduce
  89:266,target 42
  90:266,target 43
  3:0 reduce
  141:266 reduce
  28:: reduce
  141:267 reduce
  11:0,target 79
  141:268 reduce
  141:270 reduce
  141:269 reduce
  141:271 reduce
  28:? reduce
  109:/,target 95
  109:267,target 95
  110:/,target 96
  110:267,target 96
  25:257 reduce
  3:: reduce
  25:258 reduce
  25:260 reduce
  95:284,target 109
  51:267,target 59
  3:? reduce
  103:283,target 116
  25:266 reduce
  63:269,target 53
  25:267 reduce
  25:268 reduce
  94:258,target 47
  25:270 reduce
  25:269 reduce
  42:?,target 62
  25:271 reduce
  115:285,target 127
  102:257,target 98
  12:268,target 74
  43:257,target 63
  146:257 reduce
  114:260,target 123
  146:258 reduce
  146:260 reduce
  24:271,target 12
  146:261 reduce
  146:262 reduce
  55:260,target 33
  146:263 reduce
  38:0,target 58
  126:262,target 87
  146:264 reduce
  146:265 reduce
  146:266 reduce
  146:267 reduce
  28:` reduce
  138:264,target 104
  146:270 reduce
  146:271 reduce
  31:257 shift
  3:` reduce
  31:258 shift
  31:260 shift
  92:266,target 48
  100:265,target 90
  31:266 shift
  31:267 shift
  31:268 shift
  31:269 shift
  31:270 shift
  125::,target 84
  31:271 shift
  48:/,target 51
  112:267,target 125
  53:267,target 38
  102:? shift
  31:276 goto
  124:270,target 83
  96:258,target 17
  29:/ reduce
  30:/ reduce
  29:0 reduce
  30:0 reduce
  117:285,target 127
  36::,target 60
  104:257,target 98
  4:/ shift
  116:260,target 123
  29:: reduce
  30:: reduce
  36:257 reduce
  57:260,target 30
  36:258 reduce
  36:260 reduce
  29:? reduce
  30:? reduce
  97:?,target 94
  3:/,target 76
  4:: shift
  36:266 reduce
  76:/,target 29
  36:267 reduce
  36:268 reduce
  36:269 reduce
  36:270 reduce
  4:? shift
  36:271 reduce
  94:266,target 47
  33:`,target 61
  1:269,target 73
  1:270,target 73
  86:/ reduce
  114:267,target 125
  55:267,target 33
  126:270,target 87
  67:269,target 50
  29:` reduce
  30:` reduce
  98:258,target 93
  42:257 reduce
  42:258 reduce
  42:260 reduce
  106:257,target 92
  16:268,target 10
  42:266 reduce
  42:267 reduce
  42:268 reduce
  28:271,target 5
  42:269 reduce
  42:270 reduce
  42:271 reduce
  59:260,target 32
  60:260,target 35
  143:264,target 104
  103:? shift
  65:277,target 27
  96:266,target 17
  31:/ shift
  117:?,target 120
  31:0 reduce
  86:` reduce
  3:269,target 76
  3:270,target 76
  116:267,target 125
  5:/ reduce
  5:0 reduce
  31:: shift
  57:267,target 30
  128:269,target 136
  31:? shift
  69:269,target 56
  70:269,target 54
  88:`,target 41
  28:?,target 5
  5:: reduce
  108:257,target 121
  18:268,target 8
  5:? reduce
  49:257,target 55
  50:257,target 64
  121:260,target 85
  31:271,target 43
  25:0,target 20
  62:260,target 80
  145:?,target 120
  87:/ reduce
  145:264,target 104
  124:/,target 83
  11:258,target 79
  53:257 reduce
  53:258 reduce
  53:260 reduce
  98:266,target 93
  31:` reduce
  56:?,target 34
  53:266 reduce
  53:267 reduce
  106:265,target 92
  5:269,target 72
  5:270,target 72
  53:270 reduce
  35:/,target 64
  53:271 reduce
  5:` reduce
  59:267,target 32
  60:267,target 35
  72:269,target 55
  23::,target 34
  104:? shift
  21:268,target 45
  52:257,target 64
  123:260,target 82
  33:271,target 61
  58:257 reduce
  32:/ reduce
  58:258 reduce
  32:0 reduce
  58:260 reduce
  64:260,target 52
  84:?,target 27
  135:262,target 89
  87:` reduce
  140::,target 14
  58:266 reduce
  63:/,target 53
  58:267 reduce
  13:258,target 75
  58:270 reduce
  32:: reduce
  58:271 reduce
  7:0,target 69
  32:? reduce
  20:`,target 11
  19:`,target 7
  7:270,target 69
  7:269,target 69
  51::,target 53
  121:267,target 85
  62:267,target 82
  133:269,target 15
  133:270,target 15
  136:`,target 19
  74:269,target 96
  88:/ reduce
  64:257 reduce
  64:258 reduce
  64:260 reduce
  11:266,target 79
  0:260,target 7
  91:/,target 13
  113:257,target 97
  23:268,target 40
  64:266 reduce
  64:267 reduce
  54:257,target 37
  64:269 reduce
  125:260,target 84
  32:` reduce
  35:271,target 64
  66:260,target 49
  5::,target 72
  78::,target 24
  105:/ reduce
  15:258,target 6
  104:?,target 97
  105:: reduce
  9:270,target 71
  9:269,target 71
  69:257 reduce
  70:257 reduce
  105:? reduce
  69:258 reduce
  70:258 reduce
  69:260 reduce
  70:260 reduce
  123:267,target 82
  64:267,target 52
  2:`,target 77
  33:/ reduce
  69:266 reduce
  70:266 reduce
  135:270,target 89
  33:0 reduce
  69:267 reduce
  70:267 reduce
  15:?,target 3
  69:269 reduce
  70:269 reduce
  88:` reduce
  76:270,target 29
  13:266,target 75
  2:260,target 77
  7:/ reduce
  7:0 reduce
  33:: reduce
  115:257,target 121
  25:268,target 20
  56:257,target 34
  33:? reduce
  127:260,target 88
  12:0,target 74
  37:271,target 57
  7:: reduce
  68:260,target 51
  7:? reduce
  17:258,target 9
  75:257 shift
  75:258 shift
  75:260 shift
  75:261 shift
  75:262 shift
  75:263 shift
  75:264 shift
  89:/ reduce
  90:/ reduce
  101:263,target 103
  43:?,target 63
  0:267,target 9
  75:266 shift
  75:267 shift
  75:270 shift
  113:265,target 97
  22:/,target 80
  23:276,target 44
  125:267,target 84
  33:` reduce
  66:267,target 49
  39:0,target 59
  40:0,target 66
  137:269,target 141
  75:283 goto
  75:284 goto
  75:285 goto
  78:270,target 24
  10::,target 78
  106:/ reduce
  15:266,target 8
  7:` reduce
  4:260,target 7
  117:257,target 121
  27:268,target 16
  58:257,target 31
  81:257 reduce
  129:260,target 100
  130:260,target 101
  81:258 reduce
  39:271,target 59
  40:271,target 66
  81:260 reduce
  106:: reduce
  71:260,target 88
  142:262,target 102
  106:? reduce
  126::,target 87
  50:/,target 63
  81:266 reduce
  81:267 reduce
  81:270 reduce
  81:271 reduce
  20:258,target 11
  19:258,target 21
  95:264,target 104
  34:/ reduce
  34:0 reduce
  89:` reduce
  90:` reduce
  103:263,target 103
  2:267,target 77
  8:/ reduce
  37::,target 57
  115:265,target 99
  8:0 reduce
  34:: reduce
  127:267,target 88
  34:? reduce
  68:267,target 51
  140:270,target 14
  140:269,target 14
  139:269,target 39
  8:: reduce
  98:?,target 93
  86:257 reduce
  81:270,target 22
  86:258 reduce
  4:/,target 1
  86:260 reduce
  8:? reduce
  77:/,target 28
  17:266,target 9
  6:259,target 25
  86:266 reduce
  120:257,target 86
  86:267 reduce
  29:268,target 4
  30:268,target 3
  86:269 reduce
  61:257,target 36
  0:275,target 17
  91:/ reduce
  132:260,target 40
  34:`,target 65
  91:0 reduce
  42:271,target 62
  73:260,target 80
  144:262,target 102
  91:: reduce
  22:258,target 80
  34:` reduce
  97:264,target 94
  91:? reduce
  105:263,target 91
  15:274,target 28
  4:267,target 9
  8:` reduce
  92:257 reduce
  117:265,target 130
  92:258 reduce
  92:260 reduce
  129:267,target 100
  130:267,target 101
  92:266 reduce
  71:267,target 90
  92:267 reduce
  142:270,target 107
  92:269 reduce
  83:270,target 26
  107:? shift
  20:266,target 11
  19:266,target 38
  8:260,target 70
  122:257,target 135
  35:/ reduce
  32:268,target 6
  35:0 reduce
  63:257,target 53
  91:` reduce
  134:260,target 139
  44:271,target 68
  75:260,target 100
  146:262,target 103
  9:/ reduce
  9:0 reduce
  35:: reduce
  97:257 reduce
  35:? reduce
  97:258 reduce
  24:258,target 12
  97:260 reduce
  97:261 reduce
  97:262 reduce
  9:: reduce
  97:263 reduce
  89:`,target 42
  90:`,target 43
  97:264 reduce
  29:?,target 4
  30:?,target 3
  97:265 reduce
  97:266 reduce
  97:267 reduce
  9:? reduce
  97:270 reduce
  97:271 reduce
  120:265,target 86
  92:/ reduce
  73:267,target 82
  146:?,target 103
  144:270,target 102
  125:/,target 84
  22:266,target 80
  35:` reduce
  124:257,target 83
  34:268,target 65
  105:271,target 91
  65:257,target 24
  15:282,target 32
  136:260,target 19
  57:?,target 30
  46:271,target 81
  108:/ shift
  9:` reduce
  77:260,target 28
  113::,target 97
  36:/,target 60
  108:? shift
  109:263,target 95
  110:263,target 96
  8:267,target 70
  24::,target 12
  36:/ reduce
  36:0 reduce
  92:` reduce
  75:267,target 106
  146:270,target 103
  1:257,target 73
  141::,target 18
  36:: reduce
  87:269,target 44
  104:257 shift
  64:/,target 52
  104:258 shift
  104:260 shift
  104:261 shift
  24:266,target 12
  104:262 shift
  36:? reduce
  104:263 shift
  104:264 shift
  126:257,target 87
  8:0,target 70
  104:266 shift
  36:268,target 60
  104:267 shift
  67:257,target 50
  104:270 shift
  138:260,target 100
  79:260,target 25
  80:260,target 21
  104:283 goto
  93:/ reduce
  104:284 goto
  28:258,target 5
  104:285 goto
  100:261,target 90
  85:277,target 48
  112:263,target 103
  92:/,target 48
  124:265,target 83
  109:257 reduce
  110:257 reduce
  36:` reduce
  109:258 reduce
  110:258 reduce
  109:260 reduce
  110:260 reduce
  109:261 reduce
  110:261 reduce
  109:262 reduce
  110:262 reduce
  136:267,target 19
  109:263 reduce
  110:263 reduce
  109:264 reduce
  110:264 reduce
  109:265 reduce
  110:265 reduce
  77:267,target 28
  109:266 reduce
  110:266 reduce
  109:/ reduce
  109:267 reduce
  110:/ reduce
  110:267 reduce
  109:270 reduce
  110:270 reduce
  3:257,target 76
  109:271 reduce
  110:271 reduce
  79::,target 25
  80::,target 21
  89:269,target 42
  90:269,target 43
  142:285,target 127
  109:: reduce
  110:: reduce
  38:268,target 58
  109:271,target 95
  110:271,target 96
  69:257,target 56
  70:257,target 54
  105:?,target 91
  141:260,target 18
  109:? reduce
  110:? reduce
  51:271,target 61
  82:260,target 23
  37:/ reduce
  37:0 reduce
  93:` reduce
  115:257 shift
  3:`,target 76
  31:258,target 21
  115:258 shift
  102:261,target 101
  115:260 shift
  115:261 shift
  16:?,target 10
  115:262 shift
  115:263 shift
  115:264 shift
  37:: reduce
  114:263,target 103
  115:265 reduce
  115:266 shift
  115:267 shift
  115:270 shift
  37:? reduce
  115:271 reduce
  126:265,target 87
  138:267,target 106
  13:0,target 75
  133:?,target 15
  79:267,target 25
  80:267,target 21
  115:284 goto
  5:257,target 72
  112:/,target 128
  115:285 goto
  92:269,target 48
  28:266,target 5
  94:/ reduce
  41:268,target 67
  44:?,target 68
  72:257,target 55
  121:257 reduce
  143:260,target 100
  121:258 reduce
  121:260 reduce
  53:271,target 38
  121:261 reduce
  100::,target 90
  121:262 reduce
  23:/,target 33
  84:260,target 27
  121:263 reduce
  121:264 reduce
  121:265 reduce
  37:` reduce
  121:266 reduce
  121:267 reduce
  121:270 reduce
  121:271 reduce
  33:258,target 61
  41:0,target 67
  104:261,target 101
  116:263,target 103
  140:/,target 14
  11::,target 79
  111:: shift
  141:267,target 18
  51:278,target 73
  82:267,target 23
  126:257 reduce
  127::,target 88
  7:257,target 69
  126:258 reduce
  94:269,target 47
  126:260 reduce
  38:/ reduce
  126:261 reduce
  38:0 reduce
  126:262 reduce
  75:283,target 108
  126:263 reduce
  31:266,target 38
  94:` reduce
  126:264 reduce
  126:265 reduce
  126:266 reduce
  126:267 reduce
  133:257,target 15
  43:268,target 63
  126:270 reduce
  114:271,target 98
  126:271 reduce
  74:257,target 87
  145:260,target 123
  38:: reduce
  55:271,target 33
  11:257 reduce
  38::,target 58
  11:258 reduce
  86:260,target 45
  11:260 reduce
  38:? reduce
  0:257 shift
  98:262,target 93
  0:258 shift
  11:266 reduce
  0:260 shift
  11:267 reduce
  11:268 reduce
  35:258,target 64
  11:270 reduce
  11:269 reduce
  106:261,target 92
  11:271 reduce
  0:266 shift
  0:267 shift
  5:/,target 72
  0:268 shift
  0:269 shift
  0:270 shift
  78:/,target 24
  0:271 shift
  0:272 goto
  0:273 goto
  0:274 goto
  0:275 goto
  132:260 reduce
  0:276 goto
  143:267,target 106
  96:0,target 17
  0:281 goto
  35:`,target 64
  0:282 goto
  84:267,target 27
  132:269 reduce
  9:257,target 71
  38:` reduce
  96:269,target 17
  96:270,target 17
  16:257 reduce
  95:? shift
  16:258 reduce
  16:260 reduce
  33:266,target 61
  135:257,target 89
  5:257 reduce
  112:/ shift
  5:258 reduce
  16:266 reduce
  5:260 reduce
  16:267 reduce
  76:257,target 29
  16:268 reduce
  16:270 reduce
  16:269 reduce
  16:271 reduce
  57:271,target 30
  5:266 reduce
  88:260,target 41
  5:267 reduce
  5:268 reduce
  5:269 reduce
  5:270 reduce
  5:271 reduce
  112:? shift
  137:260 shift
  37:258,target 57
  63:`,target 53
  108:261,target 101
  121:263,target 85
  39:/ reduce
  40:/ reduce
  39:0 reduce
  40:0 reduce
  137:269 shift
  22:257 reduce
  22:258 reduce
  145:267,target 125
  22:260 reduce
  120:?,target 86
  39:: reduce
  40:: reduce
  86:267,target 45
  22:266 reduce
  22:267 reduce
  39:? reduce
  40:? reduce
  22:268 reduce
  138:283,target 142
  22:270 reduce
  22:269 reduce
  98:270,target 93
  22:271 reduce
  35:266,target 64
  91:`,target 13
  143:257 shift
  31:?,target 35
  143:258 shift
  143:260 shift
  78:257,target 24
  143:261 shift
  143:262 shift
  143:263 shift
  10:/,target 78
  59:271,target 32
  60:271,target 35
  143:264 shift
  91:260,target 13
  143:266 shift
  96:/ reduce
  143:267 shift
  96:0 reduce
  143:270 shift
  27:0,target 16
  39:258,target 59
  40:258,target 66
  27:257 reduce
  27:258 reduce
  27:260 reduce
  96:: reduce
  126:/,target 87
  39:` reduce
  40:` reduce
  123:263,target 82
  143:283 goto
  143:284 goto
  96:? reduce
  143:285 goto
  27:266 reduce
  27:267 reduce
  27:268 reduce
  135:265,target 89
  27:270 reduce
  27:269 reduce
  27:271 reduce
  113:/ reduce
  58:?,target 31
  88:267,target 41
  114::,target 98
  37:/,target 57
  113:: reduce
  37:266,target 57
  113:? reduce
  140:257,target 14
  121:271,target 85
  81:257,target 22
  25::,target 20
  33:257 reduce
  41:/ reduce
  102:284,target 109
  33:258 reduce
  41:0 reduce
  62:271,target 84
  33:260 reduce
  93:260,target 46
  96:` reduce
  33:266 reduce
  101:258,target 99
  33:267 reduce
  11:270,target 79
  11:269,target 79
  33:268 reduce
  33:269 reduce
  33:270 reduce
  41:: reduce
  33:271 reduce
  42:258,target 62
  113:261,target 97
  142::,target 143
  41:? reduce
  125:263,target 84
  9:0,target 71
  22:`,target 80
  91:267,target 13
  53::,target 38
  97:/ reduce
  143:283,target 145
  38:257 reduce
  38:258 reduce
  38:260 reduce
  39:266,target 59
  40:266,target 66
  142:257,target 121
  38:266 reduce
  38:267 reduce
  97:: reduce
  38:268 reduce
  123:271,target 82
  38:269 reduce
  38:270 reduce
  83:257,target 26
  38:271 reduce
  41:` reduce
  93:/,target 46
  104:284,target 109
  97:? reduce
  95:260,target 100
  114:/ reduce
  103:258,target 99
  13:270,target 75
  13:269,target 75
  50:`,target 4
  44:258,target 68
  115:261,target 101
  7::,target 69
  81::,target 22
  114:: reduce
  127:263,target 88
  114:? shift
  44:257 reduce
  44:258 reduce
  50:276,target 70
  44:260 reduce
  106:?,target 92
  42:/ reduce
  42:0 reduce
  44:266 reduce
  93:267,target 46
  44:267 reduce
  44:268 reduce
  44:269 reduce
  44:270 reduce
  44:271 reduce
  101:266,target 105
  0:271,target 13
  42:266,target 62
  42:: reduce
  144:257,target 102
  17:?,target 9
  42:? reduce
  125:271,target 84
  85:257,target 111
  97:260,target 94
  14:0,target 0
  105:258,target 91
  15:269,target 11
  15:270,target 12
  49:257 shift
  50:257 shift
  46:258,target 81
  49:258 shift
  50:258 shift
  49:260 shift
  50:260 shift
  117:261,target 101
  113:/,target 97
  98:/ reduce
  129:263,target 100
  130:263,target 101
  49:266 shift
  50:266 shift
  49:267 shift
  50:267 shift
  49:270 shift
  49:271 shift
  52:276,target 70
  50:274 goto
  98:: reduce
  50:276 goto
  42:` reduce
  49:278 goto
  50:280 goto
  24:/,target 12
  95:267,target 106
  98:? reduce
  50:282 goto
  103:266,target 105
  115:/ reduce
  2:271,target 77
  42:0,target 62
  44:266,target 68
  146:257,target 103
  127:271,target 88
  141:/,target 18
  12::,target 74
  55:257 reduce
  87:257,target 44
  55:258 reduce
  115:: reduce
  55:260 reduce
  108:284,target 126
  115:? shift
  55:266 reduce
  55:267 reduce
  0:?,target 3
  55:270 reduce
  17:270,target 9
  17:269,target 9
  55:271 reduce
  73:?,target 77
  43:/ reduce
  43:0 reduce
  120:261,target 86
  52:/,target 63
  144:265,target 102
  43:: reduce
  10:260,target 78
  43:? reduce
  39::,target 59
  40::,target 66
  97:267,target 94
  61:257 reduce
  61:258 reduce
  61:260 reduce
  105:266,target 91
  4:271,target 13
  46:266,target 81
  61:266 reduce
  61:267 reduce
  61:270 reduce
  61:271 reduce
  129:271,target 100
  130:271,target 101
  79:/,target 25
  80:/,target 21
  89:257,target 42
  90:257,target 43
  36:`,target 60
  109:258,target 95
  110:258,target 96
  20:270,target 11
  20:269,target 11
  19:269,target 41
  19:270,target 42
  43:` reduce
  51:258,target 56
  66:257 reduce
  75:263,target 103
  146:265,target 103
  66:258 reduce
  66:260 reduce
  12:260,target 74
  66:266 reduce
  66:267 reduce
  66:269 reduce
  116:? shift
  64:`,target 52
  44:/ reduce
  44:0 reduce
  92:257,target 48
  73:271,target 84
  44:: reduce
  10:267,target 78
  112:258,target 122
  121:?,target 85
  44:? reduce
  72:257 reduce
  22:270,target 80
  22:269,target 80
  72:258 reduce
  72:260 reduce
  53:258,target 38
  100:/,target 90
  124:261,target 83
  72:266 reduce
  72:267 reduce
  72:269 reduce
  92:`,target 48
  32:?,target 6
  11:/,target 79
  109:266,target 95
  110:266,target 96
  28:0,target 5
  8:271,target 70
  95:283,target 112
  51:266,target 58
  44:` reduce
  77:257 reduce
  127:/,target 88
  77:258 reduce
  77:260 reduce
  94:257,target 47
  115:284,target 126
  77:266 reduce
  12:267,target 74
  77:267 reduce
  77:270 reduce
  59:?,target 32
  60:?,target 35
  77:271 reduce
  114:258,target 122
  24:270,target 12
  24:269,target 12
  55:258,target 33
  115::,target 99
  38:/,target 58
  126:261,target 87
  117:? shift
  138:263,target 103
  16:260,target 10
  100:264,target 90
  83:257 reduce
  83:258 reduce
  83:260 reduce
  112:266,target 124
  53:266,target 38
  83:266 reduce
  83:267 reduce
  83:270 reduce
  83:271 reduce
  65:268,target 85
  66:/,target 49
  136:271,target 19
  96:257,target 17
  117:284,target 126
  77:271,target 28
  23:`,target 46
  116:258,target 122
  54::,target 37
  57:258,target 30
  88:257 reduce
  88:258 reduce
  140:`,target 14
  88:260 reduce
  18:260,target 8
  88:266 reduce
  88:267 reduce
  94:/,target 47
  88:269 reduce
  102:264,target 104
  1:268,target 73
  114:266,target 124
  55:266,target 33
  8::,target 70
  82::,target 23
  98:257,target 93
  79:271,target 25
  80:271,target 21
  16:267,target 10
  46:/ reduce
  46:0 reduce
  107:?,target 118
  94:257 reduce
  94:258 reduce
  94:260 reduce
  28:270,target 5
  28:269,target 5
  59:258,target 32
  60:258,target 35
  94:266 reduce
  46:: reduce
  94:267 reduce
  143:263,target 103
  94:269 reduce
  5:`,target 72
  46:? reduce
  18:?,target 8
  21:259,target 25
  104:264,target 104
  3:268,target 76
  15:0,target 1
  116:266,target 124
  26:277,target 48
  135:?,target 89
  57:266,target 30
  99:257 shift
  114:/,target 98
  141:271,target 18
  82:271,target 23
  46:` reduce
  18:267,target 8
  46:?,target 81
  121:258,target 85
  31:270,target 42
  31:269,target 41
  25:/,target 20
  62:258,target 79
  120:/ reduce
  145:263,target 103
  43:0,target 63
  11:257,target 79
  120:: reduce
  23:260,target 37
  13::,target 75
  98:265,target 93
  120:? reduce
  106:264,target 92
  5:268,target 72
  101:257 shift
  101:258 shift
  47:/ shift
  101:260 shift
  1:?,target 73
  101:261 shift
  101:262 shift
  101:263 shift
  101:264 shift
  59:266,target 32
  60:266,target 35
  101:266 shift
  101:267 shift
  129::,target 100
  130::,target 101
  53:/,target 38
  101:270 shift
  47:: shift
  84:271,target 27
  10:`,target 78
  101:283 goto
  101:284 goto
  123:258,target 82
  101:285 goto
  33:269,target 61
  33:270,target 61
  41::,target 67
  64:258,target 52
  135:261,target 89
  13:257,target 75
  106:257 reduce
  106:258 reduce
  106:260 reduce
  106:261 reduce
  7:/,target 69
  106:262 reduce
  25:260,target 20
  106:263 reduce
  81:/,target 22
  106:264 reduce
  106:265 reduce
  106:266 reduce
  106:267 reduce
  106:270 reduce
  108:264,target 104
  106:271 reduce
  7:268,target 69
  37:`,target 57
  121:266,target 85
  62:266,target 81
  133:268,target 15
  121:/ reduce
  145:271,target 146
  0:258,target 6
  121:: reduce
  23:267,target 39
  112:257 shift
  125:258,target 84
  112:258 shift
  121:? reduce
  35:269,target 64
  35:270,target 64
  112:260 shift
  112:261 shift
  66:258,target 49
  112:262 shift
  112:263 shift
  112:264 shift
  112:266 shift
  48:/ shift
  112:267 shift
  112:270 shift
  15:257,target 5
  27:260,target 16
  96::,target 17
  48:: shift
  112:284 goto
  112:285 goto
  9:268,target 71
  123:266,target 82
  64:266,target 52
  117:257 shift
  117:258 shift
  117:260 shift
  117:261 shift
  117:262 shift
  2:258,target 77
  117:263 shift
  117:264 shift
  117:265 shift
  93:`,target 46
  117:266 shift
  33:?,target 61
  117:267 shift
  25:267,target 20
  117:270 shift
  127:258,target 88
  12:/,target 74
  37:269,target 57
  37:270,target 57
  68:258,target 51
  29:0,target 4
  30:0,target 3
  117:284 goto
  117:285 goto
  17:257,target 9
  74:276,target 93
  29:260,target 4
  30:260,target 3
  101:262,target 102
  0:266,target 8
  113:264,target 97
  123:257 reduce
  123:258 reduce
  123:260 reduce
  123:261 reduce
  123:262 reduce
  61:?,target 36
  123:263 reduce
  125:266,target 84
  123:264 reduce
  123:265 reduce
  123:266 reduce
  66:266,target 49
  123:267 reduce
  39:/,target 59
  40:/,target 66
  123:270 reduce
  123:271 reduce
  50:/ shift
  4:258,target 21
  91:271,target 13
  27:267,target 16
  27::,target 16
  129:258,target 100
  130:258,target 101
  49:: shift
  39:269,target 59
  39:270,target 59
  40:269,target 66
  40:270,target 66
  71:258,target 65
  142:261,target 101
  49:? shift
  20:257,target 11
  19:257,target 36
  128:260 shift
  95:263,target 103
  144::,target 102
  67:/,target 50
  32:260,target 6
  103:262,target 102
  2:266,target 77
  128:269 shift
  115:264,target 104
  24:`,target 12
  13:257 reduce
  13:258 reduce
  127:266,target 88
  13:260 reduce
  128:279 goto
  55::,target 33
  68:266,target 51
  140:268,target 14
  2:257 reduce
  2:258 reduce
  13:266 reduce
  2:260 reduce
  50:280,target 71
  13:267 reduce
  13:268 reduce
  13:270 reduce
  13:269 reduce
  50:` shift
  13:271 reduce
  2:266 reduce
  141:`,target 18
  2:267 reduce
  2:268 reduce
  2:269 reduce
  2:270 reduce
  2:271 reduce
  29:267,target 4
  30:267,target 3
  123:/ reduce
  101:270,target 107
  0:274,target 16
  134:260 shift
  42:269,target 62
  42:270,target 62
  73:258,target 79
  144:261,target 102
  123:: reduce
  134:269 shift
  52:`,target 4
  22:257,target 80
  123:? reduce
  9::,target 71
  97:263,target 94
  18:257 reduce
  18:258 reduce
  83::,target 26
  18:260 reduce
  34:260,target 65
  105:262,target 91
  7:257 reduce
  4:266,target 8
  7:258 reduce
  18:266 reduce
  7:260 reduce
  18:267 reduce
  117:264,target 104
  18:268 reduce
  18:270 reduce
  18:269 reduce
  18:271 reduce
  108:?,target 120
  7:266 reduce
  129:266,target 100
  130:266,target 101
  7:267 reduce
  7:268 reduce
  7:270 reduce
  7:269 reduce
  51:: shift
  7:271 reduce
  71:266,target 89
  140:257 reduce
  140:258 reduce
  52:280,target 74
  140:260 reduce
  139:260 reduce
  51:? shift
  8:258,target 70
  140:266 reduce
  140:267 reduce
  20:?,target 11
  19:?,target 35
  140:268 reduce
  140:270 reduce
  140:269 reduce
  139:269 reduce
  140:271 reduce
  32:267,target 6
  103:270,target 107
  24:257 reduce
  44:269,target 68
  44:270,target 68
  24:258 reduce
  24:260 reduce
  75:258,target 99
  146:261,target 103
  16:0,target 10
  24:266 reduce
  136:?,target 19
  24:267 reduce
  24:268 reduce
  24:270 reduce
  24:269 reduce
  24:257,target 12
  24:271 reduce
  115:/,target 99
  36:260,target 60
  145:257 shift
  145:258 shift
  145:260 shift
  120:264,target 86
  133:0,target 15
  145:261 shift
  145:262 shift
  145:263 shift
  145:264 shift
  124:/ reduce
  0:282,target 20
  145:266 shift
  145:267 shift
  145:270 shift
  73:266,target 81
  145:271 shift
  29:257 reduce
  30:257 reduce
  124:: reduce
  29:258 reduce
  30:258 reduce
  29:260 reduce
  30:260 reduce
  44:0,target 68
  97:271,target 94
  124:? reduce
  145:284 goto
  145:285 goto
  29:266 reduce
  30:266 reduce
  34:267,target 65
  29:267 reduce
  30:267 reduce
  105:270,target 91
  29:268 reduce
  30:268 reduce
  29:270 reduce
  29:269 reduce
  15:281,target 31
  30:269 reduce
  30:270 reduce
  136:258,target 19
  29:271 reduce
  30:271 reduce
  46:269,target 81
  46:270,target 81
  52:/ shift
  77:258,target 28
  2:?,target 77
  75:?,target 97
  71:274,target 92
  26:257,target 47
  131::,target 138
  54:/,target 37
  38:260,target 58
  109:262,target 95
  110:262,target 96
  8:266,target 70
  11:`,target 79
  35:257 reduce
  35:258 reduce
  35:260 reduce
  42::,target 62
  75:266,target 105
  35:266 reduce
  35:267 reduce
  35:268 reduce
  35:269 reduce
  35:270 reduce
  35:271 reduce
  8:/,target 70
  36:267,target 60
  82:/,target 23
  138:258,target 99
  52:` shift
  79:258,target 25
  80:258,target 21
  101:285,target 110
  38:`,target 58
  125:/ reduce
  28:257,target 5
  41:257 reduce
  41:258 reduce
  100:260,target 90
  41:260 reduce
  10:271,target 78
  41:260,target 67
  112:262,target 102
  125:: reduce
  41:266 reduce
  41:267 reduce
  41:268 reduce
  41:269 reduce
  41:270 reduce
  124:264,target 83
  41:271 reduce
  125:? reduce
  136:266,target 19
  53:/ reduce
  77:266,target 28
  66:`,target 49
  71:282,target 94
  142:284,target 126
  53:: reduce
  38:267,target 58
  97::,target 94
  109:270,target 95
  110:270,target 96
  53:? reduce
  141:258,target 18
  46:257 reduce
  46:258 reduce
  46:260 reduce
  51:270,target 60
  82:258,target 23
  103:285,target 110
  46:266 reduce
  123:?,target 82
  46:267 reduce
  46:268 reduce
  46:269 reduce
  46:270 reduce
  46:271 reduce
  31:257,target 36
  102:260,target 100
  12:271,target 74
  43:260,target 63
  114:262,target 102
  94:`,target 47
  126:264,target 87
  34:?,target 65
  138:266,target 105
  13:/,target 75
  119:279,target 134
  79:266,target 25
  80:266,target 21
  52:257 shift
  52:258 shift
  31:0,target 2
  52:260 shift
  126:/ reduce
  100:267,target 90
  52:266 shift
  52:267 shift
  129:/,target 100
  130:/,target 101
  41:267,target 67
  112:270,target 107
  143:258,target 99
  126:: reduce
  52:274 goto
  53:270,target 38
  52:276 goto
  84:258,target 27
  126:? reduce
  52:280 goto
  62:?,target 77
  52:282 goto
  33:257,target 61
  41:/,target 67
  54:/ reduce
  104:260,target 100
  45:259,target 25
  116:262,target 102
  57:257 reduce
  54:: reduce
  57:258 reduce
  57:260 reduce
  141:266,target 18
  28::,target 5
  54:? reduce
  57:266 reduce
  57:267 reduce
  82:266,target 23
  57:270 reduce
  57:271 reduce
  102:267,target 106
  68:/,target 51
  43:267,target 63
  114:270,target 107
  145:258,target 122
  55:270,target 33
  86:258,target 45
  25:`,target 20
  98:261,target 93
  63:257 reduce
  56::,target 34
  63:258 reduce
  35:257,target 64
  63:260 reduce
  106:260,target 92
  16:271,target 10
  63:266 reduce
  63:267 reduce
  127:/ reduce
  63:269 reduce
  143:266,target 105
  96:/,target 17
  127:: reduce
  84:266,target 27
  127:? reduce
  96:268,target 17
  104:267,target 106
  55:/ reduce
  84::,target 27
  68:257 reduce
  116:270,target 107
  68:258 reduce
  68:260 reduce
  57:270,target 30
  88:258,target 41
  55:: reduce
  68:266 reduce
  109:?,target 95
  110:?,target 96
  68:267 reduce
  68:269 reduce
  55:? reduce
  37:257,target 57
  108:260,target 123
  18:271,target 8
  49:260,target 57
  50:260,target 66
  121:262,target 85
  7:`,target 69
  145:266,target 124
  86:266,target 45
  74:257 shift
  74:258 shift
  17:0,target 9
  74:260 shift
  74:266 shift
  106:267,target 92
  74:267 shift
  74:269 shift
  74:274 goto
  59:270,target 32
  60:270,target 35
  74:276 goto
  91:258,target 13
  112:285,target 127
  74:282 goto
  27:/,target 16
  39:257,target 59
  40:257,target 66
  52:260,target 66
  123:262,target 82
  79:257 reduce
  80:257 reduce
  79:258 reduce
  80:258 reduce
  79:260 reduce
  80:260 reduce
  135:264,target 89
  56:/ reduce
  144:/,target 102
  15::,target 2
  79:266 reduce
  80:266 reduce
  79:267 reduce
  80:267 reduce
  128:279,target 137
  79:270 reduce
  80:270 reduce
  88:266,target 41
  79:271 reduce
  80:271 reduce
  3:?,target 76
  56:: reduce
  76:?,target 29
  56:? reduce
  108:267,target 125
  55:/,target 33
  49:267,target 59
  50:267,target 68
  121:270,target 85
  102:283,target 115
  62:270,target 83
  12:`,target 74
  93:258,target 46
  114:285,target 127
  85:257 shift
  101:257,target 98
  43::,target 63
  11:268,target 79
  85:259 shift
  42:257,target 62
  113:260,target 97
  23:271,target 43
  54:260,target 37
  125:262,target 84
  9:/,target 71
  83:/,target 26
  85:277 goto
  91:266,target 13
  129:/ reduce
  130:/ reduce
  39:`,target 59
  40:`,target 66
  129:: reduce
  130:: reduce
  91:257 reduce
  52:267,target 68
  91:258 reduce
  91:260 reduce
  123:270,target 82
  129:? reduce
  130:? reduce
  104:283,target 117
  64:269,target 52
  91:266 reduce
  95:258,target 99
  91:267 reduce
  91:268 reduce
  116:285,target 127
  57:/ reduce
  91:269 reduce
  91:270 reduce
  91:271 reduce
  103:257,target 98
  13:268,target 75
  44:257,target 68
  115:260,target 123
  25:271,target 20
  56:260,target 34
  57:: reduce
  127:262,target 88
  67:`,target 50
  57:? reduce
  98::,target 93
  93:266,target 46
  96:257 reduce
  96:258 reduce
  96:260 reduce
  124:?,target 83
  0:269,target 11
  0:270,target 12
  96:266 reduce
  96:267 reduce
  96:268 reduce
  113:267,target 97
  96:269 reduce
  96:270 reduce
  96:271 reduce
  54:267,target 37
  125:270,target 84
  66:269,target 49
  35:?,target 64
  97:258,target 94
  105:257,target 91
  15:268,target 10
  46:257,target 81
  117:260,target 123
  27:271,target 16
  32:0,target 6
  58:260,target 31
  129:262,target 100
  130:262,target 101
  142:264,target 104
  131:: shift
  95:266,target 105
  58:/ reduce
  2:269,target 77
  2:270,target 77
  42:/,target 62
  115:267,target 125
  56:267,target 34
  127:270,target 88
  58:: reduce
  68:269,target 51
  58:? reduce
  29::,target 4
  30::,target 3
  103:257 shift
  17:268,target 9
  103:258 shift
  103:260 shift
  103:261 shift
  103:262 shift
  119:260,target 132
  120:260,target 86
  103:263 shift
  29:271,target 4
  30:271,target 3
  103:264 shift
  61:260,target 36
  103:266 shift
  103:267 shift
  91:?,target 13
  103:270 shift
  146::,target 103
  144:264,target 102
  69:/,target 56
  70:/,target 54
  10:258,target 78
  97:266,target 94
  103:283 goto
  103:284 goto
  103:285 goto
  105:265,target 91
  15:276,target 30
  4:270,target 12
  4:269,target 11
  57::,target 30
  117:267,target 125
  58:267,target 31
  108:257 shift
  108:258 shift
  129:270,target 100
  130:270,target 101
  108:260 shift
  108:261 shift
  108:262 shift
  71:269,target 91
  108:263 shift
  108:264 shift
  108:266 shift
  108:267 shift
  97:/,target 94
  108:270 shift
  109:257,target 95
  110:257,target 96
  20:268,target 11
  19:268,target 40
  51:257,target 55
  32:271,target 6
  63:260,target 53
  59:/ reduce
  60:/ reduce
  108:284 goto
  108:285 goto
  75:262,target 102
  146:264,target 103
  12:258,target 74
  59:: reduce
  60:: reduce
  114:257 shift
  59:? reduce
  60:? reduce
  114:258 shift
  114:260 shift
  114:261 shift
  114:262 shift
  114:263 shift
  114:264 shift
  114:265 reduce
  114:266 shift
  114:267 shift
  120:267,target 86
  114:270 shift
  114:271 reduce
  61:267,target 36
  132:269,target 40
  8:`,target 70
  22:?,target 80
  73:270,target 83
  10:266,target 78
  114:284 goto
  112:257,target 121
  114:285 goto
  22:268,target 80
  53:257,target 38
  124:260,target 83
  18:0,target 8
  34:271,target 65
  138:?,target 97
  65:259,target 25
  120:257 reduce
  120:258 reduce
  119:260 shift
  120:260 reduce
  120:261 reduce
  120:262 reduce
  120:263 reduce
  133:/ reduce
  120:264 reduce
  133:0 reduce
  120:265 reduce
  120:266 reduce
  120:267 reduce
  119:269 shift
  120:270 reduce
  120:271 reduce
  49:?,target 54
  133:: reduce
  109:265,target 95
  110:265,target 96
  105::,target 91
  28:/,target 5
  19:276,target 44
  8:270,target 70
  8:269,target 70
  119:279 goto
  133:? reduce
  63:267,target 53
  134:269,target 140
  46:0,target 81
  61:/ reduce
  75:270,target 107
  16::,target 10
  12:266,target 74
  1:260,target 73
  125:257 reduce
  114:257,target 121
  125:258 reduce
  125:260 reduce
  24:268,target 12
  61:: reduce
  125:261 reduce
  125:262 reduce
  55:257,target 33
  125:263 reduce
  125:264 reduce
  126:260,target 87
  125:265 reduce
  4:?,target 3
  36:271,target 60
  61:? reduce
  125:266 reduce
  125:267 reduce
  67:260,target 50
  77:?,target 28
  138:262,target 102
  6:277,target 27
  125:270 reduce
  125:271 reduce
  133::,target 15
  56:/,target 34
  10:257 reduce
  133:` reduce
  10:258 reduce
  16:258,target 10
  10:260 reduce
  1:0,target 73
  13:`,target 75
  10:266 reduce
  100:263,target 90
  10:267 reduce
  10:268 reduce
  10:270 reduce
  10:269 reduce
  10:271 reduce
  44::,target 68
  124:267,target 83
  136:269,target 19
  136:270,target 19
  77:270,target 28
  3:260,target 76
  84:/,target 27
  116:257,target 121
  15:257 shift
  15:258 shift
  57:257,target 30
  15:260 shift
  128:260,target 132
  38:271,target 58
  41:`,target 67
  4:257 shift
  69:260,target 56
  70:260,target 54
  4:258 shift
  15:266 shift
  4:260 shift
  15:267 shift
  15:268 shift
  15:269 shift
  15:270 shift
  15:271 shift
  4:266 shift
  18:258,target 8
  15:274 goto
  4:267 shift
  15:275 goto
  4:268 shift
  15:276 goto
  4:270 shift
  4:269 shift
  4:271 shift
  62:/ shift
  136:257 reduce
  15:281 goto
  102:263,target 103
  136:258 reduce
  15:282 goto
  136:260 reduce
  1:267,target 73
  4:276 goto
  114:265,target 98
  4:281 goto
  136:266 reduce
  136:267 reduce
  62:: shift
  136:268 reduce
  126:267,target 87
  136:269 reduce
  136:270 reduce
  136:271 reduce
  62:? shift
  67:267,target 50
  138:270,target 107
  68:`,target 51
  21:259 shift
  79:270,target 25
  80:270,target 21
  9:257 reduce
  16:266,target 10
  9:258 reduce
  5:260,target 72
  9:260 reduce
  21:268 shift
  118:257,target 131
  28:268,target 5
  100:271,target 90
  9:266 reduce
  59:257,target 32
  60:257,target 35
  9:267 reduce
  9:268 reduce
  9:270 reduce
  9:269 reduce
  41:271,target 67
  125:?,target 84
  21:277 goto
  9:271 reduce
  72:260,target 55
  143:262,target 102
  142:257 shift
  142:258 shift
  142:260 shift
  142:261 shift
  142:262 shift
  142:263 shift
  142:264 shift
  142:266 shift
  142:267 shift
  142:270 shift
  142:271 shift
  96:`,target 17
  36:?,target 60
  104:263,target 103
  3:267,target 76
  26:257 shift
  26:259 shift
  15:/,target 1
  116:265,target 129
  135:/ reduce
  142:284 goto
  142:285 goto
  33:0,target 61
  69:267,target 56
  70:267,target 54
  141:270,target 18
  141:269,target 18
  135:: reduce
  82:270,target 23
  26:277 goto
  135:? reduce
  18:266,target 8
  7:260,target 69
  121:257,target 85
  75:285,target 110
  31:268,target 40
  63:/ reduce
  62:257,target 78
  133:260,target 15
  43:271,target 63
  74:260,target 88
  145:262,target 102
  120::,target 86
  43:/,target 63
  32:257 reduce
  32:258 reduce
  32:260 reduce
  23:258,target 21
  98:264,target 93
  32:266 reduce
  32:267 reduce
  32:268 reduce
  106:263,target 92
  32:269 reduce
  32:270 reduce
  5:267,target 72
  32:271 reduce
  31::,target 34
  72:267,target 55
  143:270,target 107
  84:270,target 27
  71:/,target 86
  9:260,target 71
  123:257,target 82
  37:257 reduce
  33:268,target 61
  37:258 reduce
  63:` reduce
  37:260 reduce
  64:257,target 52
  27:`,target 16
  135:260,target 89
  37:266 reduce
  76:260,target 29
  37:267 reduce
  136:/ reduce
  37:268 reduce
  58::,target 31
  136:0 reduce
  37:269 reduce
  37:270 reduce
  37:271 reduce
  25:258,target 20
  136:: reduce
  108:263,target 103
  7:267,target 69
  136:? reduce
  98:/,target 93
  121:265,target 85
  31:276,target 44
  64:/ reduce
  133:267,target 15
  43:257 reduce
  74:267,target 90
  145:270,target 107
  43:258 reduce
  43:260 reduce
  0:257,target 5
  86:269,target 45
  43:266 reduce
  138:285,target 110
  23:266,target 38
  43:267 reduce
  43:268 reduce
  43:269 reduce
  43:270 reduce
  43:271 reduce
  125:257,target 84
  35:268,target 64
  106:271,target 92
  66:257,target 49
  112:?,target 120
  137:260,target 139
  78:260,target 24
  136:` reduce
  27:258,target 16
  9:`,target 71
  23:?,target 35
  9:267,target 71
  123:265,target 82
  64:` reduce
  135:267,target 89
  20:0,target 11
  19:0,target 7
  140:?,target 14
  76:267,target 29
  2:257,target 77
  88:269,target 41
  25:266,target 20
  127:257,target 88
  37:268,target 57
  136:0,target 19
  68:257,target 51
  140:260,target 14
  139:260,target 39
  51:?,target 54
  49:271,target 61
  81:260,target 22
  106::,target 92
  29:/,target 4
  30:/,target 3
  54:257 reduce
  54:258 reduce
  54:260 reduce
  29:258,target 4
  30:258,target 3
  101:261,target 101
  54:266 reduce
  54:267 reduce
  54:270 reduce
  54:271 reduce
  113:263,target 97
  146:/,target 103
  17::,target 9
  125:265,target 84
  5:?,target 72
  78:267,target 24
  78:?,target 24
  4:257,target 5
  57:/,target 30
  91:269,target 13
  91:270,target 13
  143:285,target 110
  27:266,target 16
  59:257 reduce
  60:257 reduce
  2:0,target 77
  59:258 reduce
  60:258 reduce
  129:257,target 100
  130:257,target 101
  59:260 reduce
  60:260 reduce
  39:268,target 59
  40:268,target 66
  71:257,target 87
  142:260,target 123
  59:266 reduce
  60:266 reduce
  59:267 reduce
  60:267 reduce
  83:260,target 26
  59:270 reduce
  60:270 reduce
  10:/ reduce
  59:271 reduce
  60:271 reduce
  10:0 reduce
  95:262,target 102
  32:258,target 6
  103:261,target 101
  10:: reduce
  115:263,target 103
  10:? reduce
  127:265,target 88
  140:267,target 14
  42:`,target 62
  49:278,target 62
  65:257 shift
  138:? shift
  65:259 shift
  81:267,target 22
  0::,target 2
  6:257,target 24
  73::,target 76
  93:269,target 46
  65:268 shift
  66:/ reduce
  145:285,target 127
  29:266,target 4
  30:266,target 3
  0:273,target 15
  42:268,target 62
  113:271,target 97
  65:277 goto
  73:257,target 78
  144:260,target 102
  54:271,target 37
  85:259,target 25
  10:` reduce
  97:262,target 94
  34:258,target 65
  69:`,target 56
  70:`,target 54
  105:261,target 91
  10:?,target 78
  71:257 shift
  71:258 shift
  117:263,target 103
  71:260 shift
  129:265,target 100
  130:265,target 101
  71:266 shift
  71:267 shift
  142:267,target 125
  71:269 shift
  126:?,target 87
  71:274 goto
  83:267,target 26
  71:276 goto
  105:/,target 91
  11:/ reduce
  8:257,target 70
  11:0 reduce
  95:270,target 107
  66:` reduce
  71:282 goto
  32:266,target 6
  44:268,target 68
  140:/ reduce
  11:: reduce
  37:?,target 57
  115:271,target 99
  140:0 reduce
  75:257,target 98
  146:260,target 103
  56:271,target 34
  16:/,target 10
  11:? reduce
  87:260,target 44
  76:257 reduce
  76:258 reduce
  76:260 reduce
  140:: reduce
  34:0,target 65
  36:258,target 60
  76:266 reduce
  140:? reduce
  76:267 reduce
  76:270 reduce
  76:271 reduce
  120:263,target 86
  133:/,target 15
  67:/ reduce
  0:281,target 19
  144:267,target 102
  121::,target 85
  11:` reduce
  44:/,target 68
  97:270,target 94
  34:266,target 65
  82:257 reduce
  82:258 reduce
  82:260 reduce
  136:257,target 19
  140:` reduce
  46:268,target 81
  77:257,target 28
  82:266 reduce
  82:267 reduce
  32::,target 6
  58:271,target 31
  82:270 reduce
  82:271 reduce
  89:260,target 42
  90:260,target 43
  38:258,target 58
  109:261,target 95
  110:261,target 96
  12:/ reduce
  12:0 reduce
  72:/,target 55
  67:` reduce
  146:267,target 103
  141:/ reduce
  28:`,target 5
  12:: reduce
  87:257 reduce
  141:0 reduce
  87:258 reduce
  87:260 reduce
  87:267,target 44
  12:? reduce
  59::,target 32
  60::,target 35
  87:266 reduce
  87:267 reduce
  141:: reduce
  87:269 reduce
  36:266,target 60
  141:? reduce
  138:257,target 98
  120:271,target 86
  79:257,target 25
  80:257,target 21
  101:284,target 109
  61:271,target 36
  68:/ reduce
  92:260,target 48
  100:258,target 90
  10:270,target 78
  10:269,target 78
  41:258,target 67
  112:261,target 101
  93:257 reduce
  12:` reduce
  93:258 reduce
  93:260 reduce
  124:263,target 83
  4:281,target 23
  93:266 reduce
  93:267 reduce
  93:269 reduce
  141:` reduce
  113:?,target 97
  89:267,target 42
  90:267,target 43
  38:266,target 58
  141:257,target 18
  24:?,target 12
  95:285,target 110
  13:/ reduce
  82:257,target 23
  13:0 reduce
  103:284,target 109
  68:` reduce
  98:257 reduce
  94:260,target 47
  98:258 reduce
  98:260 reduce
  98:261 reduce
  98:262 reduce
  98:263 reduce
  98:264 reduce
  102:258,target 99
  13:: reduce
  98:265 reduce
  12:270,target 74
  12:269,target 74
  98:266 reduce
  141:?,target 18
  98:267 reduce
  43:258,target 63
  98:270 reduce
  114:261,target 101
  13:? reduce
  98:271 reduce
  120:/,target 86
  126:263,target 87
  142:: shift
  142:? shift
  92:267,target 48
  31:/,target 33
  69:/ reduce
  70:/ reduce
  100:266,target 90
  41:266,target 67
  100:257 reduce
  100:258 reduce
  143:257,target 98
  100:260 reduce
  100:261 reduce
  100:262 reduce
  13:` reduce
  100:263 reduce
  124:271,target 83
  84:257,target 27
  100:264 reduce
  18::,target 8
  100:265 reduce
  100:266 reduce
  100:267 reduce
  96:260,target 17
  100:270 reduce
  100:271 reduce
  104:258,target 99
  79:?,target 25
  80:?,target 21
  116:261,target 101
  135::,target 89
  58:/,target 31
  3:0,target 76
  15:`,target 4
  105:257 reduce
  14:0 accept
  105:258 reduce
  105:260 reduce
  46::,target 81
  69:` reduce
  70:` reduce
  94:267,target 47
  105:261 reduce
  105:262 reduce
  105:263 reduce
  105:264 reduce
  105:265 reduce
  102:266,target 105
  105:266 reduce
  105:267 reduce
  1:271,target 73
  105:270 reduce
  43:266,target 63
  105:271 reduce
  145:257,target 121
  126:271,target 87
  86:257,target 45
  86:/,target 45
  98:260,target 93
  143:? shift
  106:258,target 92
  43:`,target 63
  16:270,target 10
  16:269,target 10
  1::,target 73
  71:/ shift
  100:?,target 90
  96:267,target 17
  104:266,target 105
  3:271,target 76
  71:`,target 4
  11:?,target 79
  88:257,target 41
  116:257 shift
  116:258 shift
  116:260 shift
  116:261 shift
  116:262 shift
  108:258,target 122
  116:263 shift
  116:264 shift
  127:?,target 88
  18:270,target 8
  18:269,target 8
  15:/ shift
  116:265 shift
  15:0 reduce
  116:266 shift
  49:258,target 56
  50:258,target 65
  116:267 shift
  71:` shift
  121:261,target 85
  106:/,target 92
  116:270 shift
  144:/ reduce
  15:: shift
  15:? shift
  11:260,target 79
  38:?,target 58
  116:284 goto
  116:285 goto
  17:/,target 9
  144:: reduce
  98:267,target 93
  144:? reduce
  106:266,target 92
  5:271,target 72
  35:0,target 64
  122:257 shift
  72:/ reduce
  91:257,target 13
  112:284,target 126
  15:` shift
  52:258,target 65
  123:261,target 82
  135:263,target 89
  13:260,target 75
  127:257 reduce
  127:258 reduce
  127:260 reduce
  33::,target 61
  127:261 reduce
  127:262 reduce
  127:263 reduce
  127:264 reduce
  127:265 reduce
  127:266 reduce
  127:267 reduce
  108:266,target 124
  127:270 reduce
  7:271,target 69
  127:271 reduce
  16:/ reduce
  49:266,target 58
  50:266,target 67
  16:0 reduce
  12:257 reduce
  72:` reduce
  12:258 reduce
  0:/,target 1
  12:260 reduce
  73:/,target 95
  133:271,target 15
  93:257,target 46
  1:257 reduce
  114:284,target 126
  1:258 reduce
  16:: reduce
  12:266 reduce
  1:260 reduce
  12:267 reduce
  12:268 reduce
  12:270 reduce
  12:269 reduce
  11:267,target 79
  12:271 reduce
  91:0,target 13
  16:? reduce
  29:`,target 4
  30:`,target 3
  1:266 reduce
  113:258,target 97
  1:267 reduce
  23:269,target 41
  23:270,target 42
  1:268 reduce
  1:269 reduce
  1:270 reduce
  1:271 reduce
  54:258,target 37
  61::,target 36
  125:261,target 84
  133:257 reduce
  133:258 reduce
  133:260 reduce
  145:? shift
  133:266 reduce
  133:267 reduce
  133:268 reduce
  133:269 reduce
  133:270 reduce
  15:260,target 7
  73:/ shift
  133:271 reduce
  17:257 reduce
  17:258 reduce
  17:260 reduce
  6:257 shift
  73:: shift
  21:277,target 27
  17:266 reduce
  9:271,target 71
  6:259 shift
  17:267 reduce
  16:` reduce
  17:268 reduce
  52:266,target 67
  17:270 reduce
  17:269 reduce
  73:? shift
  17:271 reduce
  6:268 shift
  135:271,target 89
  95:257,target 98
  116:284,target 126
  138:257 shift
  76:271,target 29
  138:258 shift
  138:260 shift
  138:261 shift
  138:262 shift
  13:267,target 75
  6:277 goto
  138:263 shift
  138:264 shift
  114:?,target 120
  115:258,target 122
  138:266 shift
  138:267 shift
  25:270,target 20
  25:269,target 20
  138:270 shift
  56:258,target 34
  127:261,target 88
  23:257 shift
  23:258 shift
  23:260 shift
  50:274,target 69
  17:/ reduce
  17:0 reduce
  25:?,target 20
  138:283 goto
  138:284 goto
  138:285 goto
  23:266 shift
  17:260,target 9
  23:267 shift
  23:268 shift
  23:269 shift
  23:270 shift
  23:271 shift
  101:264,target 104
  146:/ reduce
  17:: reduce
  0:268,target 10
  23:276 goto
  113:266,target 97
  144:257 reduce
  17:? reduce
  144:258 reduce
  144:260 reduce
  142:?,target 120
  144:261 reduce
  54:266,target 37
  144:262 reduce
  144:263 reduce
  146:: reduce
  144:264 reduce
  144:265 reduce
  121:/,target 85
  144:266 reduce
  144:267 reduce
  97:257,target 94
  146:? reduce
  144:270 reduce
  144:271 reduce
  78:271,target 24
  28:257 reduce
  15:267,target 9
  28:258 reduce
  28:260 reduce
  53:?,target 38
  74:/ shift
  117:258,target 122
  27:270,target 16
  27:269,target 16
  32:/,target 6
  58:258,target 31
  28:266 reduce
  129:261,target 100
  130:261,target 101
  28:267 reduce
  28:268 reduce
  28:270 reduce
  28:269 reduce
  28:271 reduce
  142:263,target 103
  52:274,target 69
  17:` reduce
  20:260,target 11
  19:260,target 37
  20::,target 11
  19::,target 34
  103:264,target 104
  2:268,target 77
  115:266,target 124
  7:?,target 69
  81:?,target 22
  56:266,target 34
  34:257 reduce
  34:258 reduce
  34:260 reduce
  136::,target 19
  59:/,target 32
  60:/,target 35
  140:271,target 14
  99:257,target 113
  50:282,target 72
  34:266 reduce
  34:267 reduce
  81:271,target 22
  34:268 reduce
  18:/ reduce
  34:269 reduce
  34:270 reduce
  18:0 reduce
  34:271 reduce
  17:267,target 9
  16:`,target 10
  74:` shift
  120:258,target 86
  29:270,target 4
  29:269,target 4
  30:269,target 3
  30:270,target 3
  47::,target 50
  61:258,target 36
  0:276,target 18
  18:: reduce
  144:263,target 102
  18:? reduce
  10:257,target 78
  133:`,target 15
  22:260,target 80
  97:265,target 94
  87:/,target 44
  39:257 reduce
  40:257 reduce
  39:258 reduce
  40:258 reduce
  39:260 reduce
  40:260 reduce
  105:264,target 91
  15:275,target 29
  4:268,target 10
  39:266 reduce
  40:266 reduce
  117:266,target 124
  39:267 reduce
  40:267 reduce
  44:`,target 68
  39:268 reduce
  40:268 reduce
  39:269 reduce
  39:270 reduce
  40:269 reduce
  40:270 reduce
  39:271 reduce
  40:271 reduce
  58:266,target 31
  2::,target 77
  142:271,target 144
  52:282,target 72
  83:271,target 26
  18:` reduce
  75:? shift
  20:267,target 11
  19:267,target 39
  101:?,target 97
  32:269,target 6
  32:270,target 6
  63:258,target 53
  75:261,target 101
  146:263,target 103
  45:259 shift
  12:257,target 74
  72:`,target 55
  12:?,target 74
  24:260,target 12
  20:/ reduce
  19:/ shift
  6:268,target 26
  45:277 goto
  20:0 reduce
  19:0 reduce
  120:266,target 86
  61:266,target 36
  20:: reduce
  19:: shift
  144:271,target 102
  20:? reduce
  19:? shift
  51:257 shift
  51:258 shift
  22:267,target 80
  39:?,target 59
  40:?,target 66
  51:260 shift
  124:258,target 83
  18:/,target 8
  34:269,target 65
  34:270,target 65
  51:266 shift
  51:267 shift
  4:276,target 22
  51:270 shift
  51:271 shift
  36:0,target 60
  76:/ reduce
  51:278 goto
  71:276,target 93
  26:259,target 25
  135:/,target 89
  76:: reduce
  109:264,target 95
  110:264,target 96
  20:` reduce
  19:` reduce
  8:268,target 70
  76:? reduce
}

array set ::snip_rules {
  98,l 284
  9,l 273
  11,l 273
  32,l 278
  53,l 280
  74,l 281
  95,l 283
  6,l 273
  28,l 278
  50,l 280
  49,l 280
  71,l 281
  92,l 283
  3,l 273
  25,l 278
  46,l 280
  67,l 281
  0,l 286
  88,l 283
  22,l 278
  43,l 280
  64,l 281
  85,l 283
  18,l 276
  40,l 279
  39,l 279
  61,l 281
  82,l 283
  15,l 275
  36,l 278
  102,l 285
  57,l 281
  78,l 281
  99,l 284
  12,l 274
  33,l 278
  54,l 280
  75,l 281
  96,l 283
  7,l 273
  29,l 278
  30,l 278
  51,l 280
  72,l 281
  93,l 283
  4,l 273
  26,l 278
  47,l 280
  68,l 281
  90,l 283
  89,l 283
  1,l 272
  23,l 278
  44,l 280
  65,l 281
  86,l 283
  19,l 276
  20,l 277
  41,l 280
  62,l 281
  83,l 283
  16,l 276
  37,l 278
  103,l 285
  58,l 281
  80,l 281
  79,l 281
  13,l 274
  34,l 278
  100,l 284
  55,l 280
  76,l 281
  97,l 283
  8,l 273
  10,l 273
  31,l 278
  52,l 280
  73,l 281
  94,l 283
  5,l 273
  27,l 278
  48,l 280
  70,l 281
  69,l 281
  91,l 283
  2,l 273
  24,l 278
  45,l 280
  66,l 281
  87,l 283
  21,l 278
  42,l 280
  63,l 281
  84,l 283
  17,l 276
  38,l 278
  60,l 281
  59,l 281
  81,l 282
  14,l 275
  35,l 278
  101,l 284
  56,l 280
  77,l 281
}

array set ::snip_rules {
  63,dc 2
  12,dc 2
  77,dc 1
  26,dc 2
  100,dc 3
  92,dc 1
  3,dc 2
  41,dc 2
  55,dc 1
  70,dc 1
  69,dc 1
  18,dc 10
  84,dc 2
  33,dc 1
  98,dc 2
  9,dc 1
  47,dc 2
  62,dc 2
  11,dc 1
  76,dc 1
  25,dc 2
  91,dc 1
  2,dc 2
  40,dc 1
  39,dc 2
  54,dc 1
  68,dc 2
  17,dc 6
  83,dc 2
  32,dc 1
  97,dc 2
  8,dc 1
  46,dc 2
  61,dc 2
  10,dc 1
  75,dc 1
  24,dc 2
  90,dc 1
  89,dc 3
  1,dc 1
  38,dc 1
  53,dc 1
  67,dc 2
  16,dc 2
  82,dc 2
  31,dc 1
  96,dc 1
  7,dc 1
  45,dc 2
  60,dc 2
  59,dc 2
  74,dc 1
  23,dc 2
  0,dc 1
  88,dc 2
  37,dc 1
  52,dc 1
  66,dc 2
  15,dc 9
  81,dc 3
  29,dc 2
  30,dc 1
  103,dc 8
  95,dc 1
  6,dc 2
  44,dc 2
  58,dc 2
  73,dc 1
  22,dc 2
  87,dc 2
  36,dc 1
  51,dc 1
  65,dc 2
  14,dc 10
  80,dc 1
  79,dc 1
  28,dc 2
  102,dc 6
  94,dc 1
  5,dc 2
  43,dc 2
  57,dc 2
  72,dc 1
  21,dc 2
  86,dc 2
  35,dc 1
  50,dc 1
  49,dc 1
  64,dc 2
  13,dc 6
  78,dc 1
  27,dc 2
  101,dc 3
  93,dc 1
  4,dc 2
  42,dc 2
  56,dc 1
  71,dc 1
  19,dc 9
  20,dc 1
  85,dc 2
  34,dc 1
  99,dc 2
  48,dc 2
}

array set ::snip_rules {
  41,line 271
  96,line 442
  7,line 107
  37,line 255
  93,line 433
  4,line 98
  34,line 246
  90,line 424
  89,line 421
  1,line 87
  102,line 464
  31,line 237
  86,line 412
  27,line 225
  83,line 403
  24,line 216
  80,line 390
  79,line 387
  21,line 207
  76,line 378
  17,line 159
  73,line 369
  14,line 144
  70,line 360
  69,line 357
  11,line 119
  66,line 348
  63,line 339
  60,line 330
  59,line 327
  56,line 316
  53,line 307
  50,line 298
  49,line 295
  46,line 286
  43,line 277
  98,line 450
  9,line 113
  40,line 266
  39,line 263
  95,line 439
  6,line 104
  36,line 252
  92,line 430
  3,line 95
  33,line 243
  88,line 418
  101,line 459
  29,line 231
  30,line 234
  85,line 409
  26,line 222
  82,line 400
  23,line 213
  78,line 384
  19,line 169
  20,line 202
  75,line 375
  16,line 156
  72,line 366
  13,line 131
  68,line 354
  10,line 116
  65,line 345
  62,line 336
  58,line 324
  55,line 313
  52,line 304
  48,line 292
  45,line 283
  42,line 274
  97,line 445
  8,line 110
  38,line 258
  94,line 436
  5,line 101
  35,line 249
  91,line 427
  2,line 92
  103,line 467
  32,line 240
  87,line 415
  100,line 456
  28,line 228
  84,line 406
  25,line 219
  81,line 395
  22,line 210
  77,line 381
  18,line 166
  74,line 372
  15,line 151
  71,line 363
  12,line 128
  67,line 351
  64,line 342
  61,line 333
  57,line 321
  54,line 310
  51,line 301
  47,line 289
  44,line 280
  99,line 453
}

proc snip_parse {} {
    set snip_state_stack {0}
    set snip_value_stack {{}}
    set snip_token ""
    set snip_accepted 0
    while {$snip_accepted == 0} {
        set snip_state [lindex $snip_state_stack end]
        if {$snip_token == ""} {
            set ::snip_lval ""
            set snip_token [snip_lex]
            set snip_buflval $::snip_lval
        }
        if {![info exists ::snip_table($snip_state:$snip_token)]} {
            # pop off states until error token accepted
            while {[llength $snip_state_stack] > 0 && \
                       ![info exists ::snip_table($snip_state:error)]} {
                set snip_state_stack [lrange $snip_state_stack 0 end-1]
                set snip_value_stack [lrange $snip_value_stack 0 \
                                       [expr {[llength $snip_state_stack] - 1}]]
                set snip_state [lindex $snip_state_stack end]
            }
            if {[llength $snip_state_stack] == 0} {
                snip_error "parse error"
                return 1
            }
            lappend snip_state_stack [set snip_state $::snip_table($snip_state:error,target)]
            lappend snip_value_stack {}
            # consume tokens until it finds an acceptable one
            while {![info exists ::snip_table($snip_state:$snip_token)]} {
                if {$snip_token == 0} {
                    snip_error "end of file while recovering from error"
                    return 1
                }
                set ::snip_lval {}
                set snip_token [snip_lex]
                set snip_buflval $::snip_lval
            }
            continue
        }
        switch -- $::snip_table($snip_state:$snip_token) {
            shift {
                lappend snip_state_stack $::snip_table($snip_state:$snip_token,target)
                lappend snip_value_stack $snip_buflval
                set snip_token ""
            }
            reduce {
                set snip_rule $::snip_table($snip_state:$snip_token,target)
                set snip_l $::snip_rules($snip_rule,l)
                if {[info exists ::snip_rules($snip_rule,e)]} {
                    set snip_dc $::snip_rules($snip_rule,e)
                } else {
                    set snip_dc $::snip_rules($snip_rule,dc)
                }
                set snip_stackpointer [expr {[llength $snip_state_stack]-$snip_dc}]
                snip_setupvalues $snip_value_stack $snip_stackpointer $snip_dc
                set _ $1
                set ::snip_lval [lindex $snip_value_stack end]
                switch -- $snip_rule {
                    1 { 
        set ::snip_value $1
       }
                    2 { 
           set _ [concat $1 [list $2 {}]]
          }
                    3 { 
           set _ [concat $1 [list $2 {}]]
          }
                    4 { 
           set _ [concat $1 $2]
          }
                    5 { 
           set _ [concat $1 $2]
          }
                    6 { 
           set _ [concat $1 [list $2 {}]]
          }
                    7 { 
           set _ [list $1 {}]
          }
                    8 { 
           set _ [list $1 {}]
          }
                    9 { 
           set _ $1
          }
                    10 { 
           set _ $1
          }
                    11 { 
           set _ [list $1 {}]
          }
                    12 { 
           if {[set val [snippets::get_tabstop $::snip_txtt $2]] ne ""} {
             set _ [list $val {}]
           } else {
             set _ [list "\$$2" [snippets::set_tabstop $::snip_txtt $2]]
           }
          }
                    13 { 
           set _ [apply_tabstop $5 [snippets::set_tabstop $::snip_txtt $3 $5]]
          }
                    14 { 
             if {[set val [snippets::get_tabstop $::snip_txtt $3]] ne ""} {
               set regexp_opts [list]
               if {[string first g $9] != -1} {
                 lappend regexp_opts -all
               }
               set _ [list [parse_format $7 [regexp -inline {*}$regexp_opts -- $5 $val]] [list]]
             } else {
               set _ [list [get_retval $1 $2 $3 $4 $5 $6 $7 $8 $9 $10] [snippets::set_tabstop $::snip_txtt $3]]
             }
            }
                    15 { 
             if {[set val [snippets::get_tabstop $::snip_txtt $3]] ne ""} {
               set _ [list [parse_format $7 [regexp -inline -- $5 $val]] [list]]
             } else {
               set _ [list [get_retval $1 $2 $3 $4 $5 $6 $7 $8 $9] [snippets::set_tabstop $::snip_txtt $3]]
             }
            }
                    16 { 
            set _ $2
           }
                    17 { 
            set _ [expr {($3 eq "") ? [lindex $5 0] : $3}]
           }
                    18 { 
            set regexp_opts [list]
            if {[string first g $9] != -1} {
              lappend regexp_opts "-all"
            }
            set _ [parse_format $7 [regexp -inline {*}$regexp_opts -- $5 $3]]
           }
                    19 { 
            set _ [parse_format $7 [regexp -inline -- $5 $3]]
           }
                    20 { 
           set txtt $::snip_txtt
           switch -glob $1 {
             SELECTED_TEXT   { set _ [expr {![catch { $txtt get sel.first sel.last } rc] ? $rc : ""}] }
             CLIPBOARD       { set _ [expr {![catch "clipboard get" rc] ? $rc : ""}] }
             CURRENT_LINE    { set _ [$txtt get "insert linestart" "insert lineend"] }
             CURRENT_WORD    { set _ [$txtt get "insert wordstart" "insert wordend"] }
             DIRECTORY       { set _ [file dirname [gui::get_info {} current fname]] }
             FILEPATH        { set _ [gui::get_info {} current fname] }
             FILENAME        { set _ [file tail [gui::get_info {} current fname]] }
             LINE_INDEX      { set _ [lindex [split [$txtt index insert] .] 1] }
             LINE_NUMBER     { set _ [lindex [split [$txtt index insert] .] 0] }
             CURRENT_DATE    { set _ [clock format [clock seconds] -format "%m/%d/%Y"] }
             CURRENT_DATE2   { set _ [clock format [clock seconds] -format "%Y/%m/%d"] }
             CURRENT_TIME    { set _ [clock format [clock seconds] -format "%I:%M %p"] }
             CURRENT_MON     { set _ [clock format [clock seconds] -format "%b"] }
             CURRENT_MONTH   { set _ [clock format [clock seconds] -format "%B"] }
             CURRENT_MON1    { set _ [clock format [clock seconds] -format "%N"] }
             CURRENT_MON2    { set _ [clock format [clock seconds] -format "%m"] }
             CURRENT_DAYN    { set _ [clock format [clock seconds] -format "%a"] }
             CURRENT_DAYNAME { set _ [clock format [clock seconds] -format "%A"] }
             CURRENT_DAY1    { set _ [clock format [clock seconds] -format "%e"] }
             CURRENT_DAY2    { set _ [clock format [clock seconds] -format "%d"] }
             CURRENT_YEAR2   { set _ [clock format [clock seconds] -format "%y"] }
             CURRENT_YEAR    { set _ [clock format [clock seconds] -format "%Y"] }
             CLIPHIST*       {
               regexp {\[(\d+)\]} $1 -> index
               set _ [lindex [cliphist::get_history] $index 1]
             }
           }
          }
                    21 { 
           set _ "$1$2"
          }
                    22 { 
           set _ "$1\n"
          }
                    23 { 
           set _ "$1\t"
          }
                    24 { 
           set _ "$1$2"
          }
                    25 { 
           set _ "$1\$"
          }
                    26 { 
           set _ "$1\("
          }
                    27 { 
           set _ "$1)"
          }
                    28 { 
           set _ "?"
          }
                    29 { 
           set _ ":"
          }
                    30 { 
           set _ $1
          }
                    31 { 
           set _ "\n"
          }
                    32 { 
           set _ "\t"
          }
                    33 { 
           set _ $1
          }
                    34 { 
           set _ "\$"
          }
                    35 { 
           set _ "("
          }
                    36 { 
           set _ ")"
          }
                    37 { 
           set _ "?"
          }
                    38 { 
           set _ ":"
          }
                    39 { 
        set _ "$1$2"
       }
                    40 { 
        set _ $1
       }
                    41 { 
         set _ [merge_values $1 [list $2 {}]]
        }
                    42 { 
         set _ [merge_values $1 [list "\n" {}]]
        }
                    43 { 
         set _ [merge_values $1 [list "\t" {}]]
        }
                    44 { 
         set _ [merge_values $1 [list $2 {}]]
        }
                    45 { 
         set _ [merge_values $1 [list "/" {}]]
        }
                    46 { 
         set _ [merge_values $1 [list $2 {}]]
        }
                    47 { 
         set _ [merge_values $1 [list $2 {}]]
        }
                    48 { 
         set _ [concat $1 $2]
        }
                    49 { 
         set _ [list $1 {}]
        }
                    50 { 
         set _ [list "\n" {}]
        }
                    51 { 
         set _ [list "\t" {}]
        }
                    52 { 
         set _ [list $1 {}]
        }
                    53 { 
         set _ [list "/" {}]
        }
                    54 { 
         set _ [list $1 {}]
        }
                    55 { 
         set _ [list $1 {}]
        }
                    56 { 
         set _ $1
        }
                    57 { 
        set _ "$1$2"
       }
                    58 { 
        set _ "$1\\n"
       }
                    59 { 
        set _ "$1\\t"
       }
                    60 { 
        set _ "$1$2"
       }
                    61 { 
        set _ "$1/"
       }
                    62 { 
        set _ "$1\("
       }
                    63 { 
        set _ "$1)"
       }
                    64 { 
        set _ "$1?"
       }
                    65 { 
        set _ "$1:"
       }
                    66 { 
        set _ "$1$2"
       }
                    67 { 
        set _ "$1$2"
       }
                    68 { 
        set _ "$1$2"
       }
                    69 { 
        set _ $1
       }
                    70 { 
        set _ "\\n"
       }
                    71 { 
        set _ "\\t"
       }
                    72 { 
        set _ $1
       }
                    73 { 
        set _ "/"
       }
                    74 { 
        set _ "("
       }
                    75 { 
        set _ ")"
       }
                    76 { 
        set _ "?"
       }
                    77 { 
        set _ ":"
       }
                    78 { 
        set _ $1
       }
                    79 { 
        set _ $1
       }
                    80 { 
        set _ $1
       }
                    81 { 
         set _ [expr {![catch "exec $2" rc] ? $rc : ""}]
        }
                    82 { 
          set _ "$1$2"
         }
                    83 { 
          set _ "$1\n"
         }
                    84 { 
          set _ "$1\t"
         }
                    85 { 
          set _ "$1$2"
         }
                    86 { 
          set _ "$1?"
         }
                    87 { 
          set _ "$1$2"
         }
                    88 { 
          set _ "$1$2"
         }
                    89 { 
          set _ "$1\$$3"
         }
                    90 { 
          set _ $1
         }
                    91 { 
          set _ "\n"
         }
                    92 { 
          set _ "\t"
         }
                    93 { 
          set _ $1
         }
                    94 { 
          set _ "?"
         }
                    95 { 
          set _ $1
         }
                    96 { 
          set _ $1
         }
                    97 { 
          set _ "\$$2"
         }
                    98 { 
              set _ "$1$2"
             }
                    99 { 
              set _ "$1$2"
             }
                    100 { 
              set _ "$1$2$3"
             }
                    101 { 
              set _ "$1$2$3"
             }
                    102 { 
               set _ [get_retval $1 $2 $3 $4 $5 $6]
              }
                    103 { 
               set _ [get_retval $1 $2 $3 $4 $5 $6 $7 $8]
              }
                }
                snip_unsetupvalues $snip_dc
                # pop off tokens from the stack if normal rule
                if {![info exists ::snip_rules($snip_rule,e)]} {
                    incr snip_stackpointer -1
                    set snip_state_stack [lrange $snip_state_stack 0 $snip_stackpointer]
                    set snip_value_stack [lrange $snip_value_stack 0 $snip_stackpointer]
                }
                # now do the goto transition
                lappend snip_state_stack $::snip_table([lindex $snip_state_stack end]:$snip_l,target)
                lappend snip_value_stack $_
            }
            accept {
                set snip_accepted 1
            }
            goto -
            default {
                puts stderr "Internal parser error: illegal command $::snip_table($snip_state:$snip_token)"
                return 2
            }
        }
    }
    return 0
}

######
# end autogenerated taccle functions
######

rename snip_error snip_error_orig
proc snip_error {s} {
  set ::snip_errstr "[string repeat " " $::snip_begpos]^"
  set ::snip_errmsg $s
}

